
retrive_simulation_parameters <- function() {

  config <- yaml::read_yaml(here::here("config.yaml"))

  res_paths <- list.files(config$results_folder, full.names = TRUE)
  res_files <- basename(res_paths)

  res_tibs <- list()
  for (i in seq_along(res_files)) {

    file <- res_files[[i]]
    full_filename <- file

    # Remove .rds from end of file.
    file <- substr(file, 1, nchar(file) - 4)

    split_file <- strsplit(file, "-")[[1]]

    sim_id <- split_file[[1]]
    method_id <- split_file[[2]]

    method_par_path <- paste0(here::here(config$method_pars_folder, method_id), ".json")
    sim_par_path <- paste0(here::here(config$sim_pars_folder, sim_id), ".json")

    method_pars <- jsonlite::fromJSON(method_par_path)
    sim_pars_raw <- jsonlite::fromJSON(sim_par_path)

    # This converts 'true' vectors in the list into sublists.
    sim_pars <- list()
    for (name in names(sim_pars_raw)) {
      if (length(sim_pars_raw[[name]]) > 1) {
        sim_pars[[name]] <- list(sim_pars_raw[[name]])
      } else {
        sim_pars[[name]] <- sim_pars_raw[[name]]
      }
    }

    sim_pars_tib <- tibble::as_tibble(sim_pars)
    method_pars_tib <- tibble::as_tibble(method_pars)
    # Add the full list of parameters for backwards compatibility.
    method_pars_tib <- tibble(method_pars_tib, pars = method_id)

    sim_pars_tib <- dplyr::rename(
      sim_pars_tib,
      sim_label = label,
      sim_name = file_name
    )
    method_pars_tib <- dplyr::rename(method_pars_tib, method_name = file_name)

    all_pars_tib <- dplyr::bind_cols(sim_pars_tib, method_pars_tib)
    all_pars_tib <- tibble::add_column(all_pars_tib, file_name = full_filename)

    res_tibs[[i]] <- all_pars_tib
  }

  out <- dplyr::bind_rows(res_tibs)

  out
}
