# Documentation TODO
find_unique_marker_genes <- function(sce) {

  stopifnot(is(sce, "SingleCellExperiment"))

  n_groups <- length(unique(colData(sce)$Group))
  col_names <- paste0("DEFacGroup", 1:n_groups)
  data <- as.matrix(rowData(sce)[, col_names])

  out <- list()
  for (i in 1:n_groups) {
    not_i <- setdiff(1:n_groups, i)
    tmp_genes <- numeric(10000)
    tmp_fcs <- numeric(10000)
    for (j in 1:nrow(data)) {
      slice <- data[j, not_i]
      if (data[j, i] != 1 && all(slice == 1)) {
        tmp_genes[[j]] <- j
        tmp_fcs[[j]] <- data[j, i]
      }
    }

    # Gene id cannot be 0.
    mask <- tmp_genes == 0
    tmp_genes <- tmp_genes[!mask]
    tmp_fcs <- tmp_fcs[!mask]
    # Add "Gene" to match other ids.
    tmp_genes <- paste0("Gene", tmp_genes)
    out[[i]] <- data.frame(gene = tmp_genes, fc = tmp_fcs)
  }

  names(out) <- paste0("group_", 1:n_groups)
  out
}

# Documentation TODO
find_semi_unique_marker_genes <- function(sce) {

  stopifnot(is(sce, "SingleCellExperiment"))

  n_groups <- length(unique(colData(sce)$Group))
  col_names <- paste0("DEFacGroup", 1:n_groups)
  data <- as.matrix(rowData(sce)[, col_names])

  # K is the maximum number of other groups a gene can be expressed in and
  # still be counted as a semi-unique marker gene. Currently it is defined as
  # 20% of the total number of groups. Though that number is somewhat arbitrary.
  K <- 0.2 * n_groups

  # FIXME: We make the assumption that the number of genes is less than 10000.
  out <- list()
  for (i in 1:n_groups) {
    not_i <- setdiff(1:n_groups, i)
    tmp_genes <- numeric(10000)
    tmp_fcs <- numeric(10000)
    tmp_other_groups <- list()
    tmp_other_fcs <- list()
    for (j in 1:nrow(data)) {
      slice <- data[j, not_i]
      n_de <- sum(slice != 1)
      if (data[j, i] != 1 && n_de >= 1 && n_de <= K) {
        tmp_genes[[j]] <- j
        tmp_fcs[[j]] <- data[j, i]

        other_groups <- setdiff(which(data[j, ] != 1), i)
        tmp_other_groups[[j]] <- other_groups
        tmp_other_fcs[[j]] <- as.numeric(data[j, other_groups])
      }
    }

    # Gene id cannot be 0.
    mask <- tmp_genes == 0
    tmp_genes <- tmp_genes[!mask]
    tmp_fcs <- tmp_fcs[!mask]
    tmp_other_groups <- tmp_other_groups[!mask]
    tmp_other_fcs <- tmp_other_fcs[!mask]

    # Add "Gene" to match other ids.
    tmp_genes <- paste0("Gene", tmp_genes)
    out[[i]] <- tibble::tibble(
      gene = tmp_genes,
      fc = tmp_fcs,
      other_groups = tmp_other_groups,
      other_fcs = tmp_other_fcs
    )
  }

  names(out) <- paste0("group_", 1:n_groups)
  out
}
