# This file specifies the marker genes methods (and their parameters) which
# will be assessed.

suppressMessages({
  library(tibble)
  library(tidyr)
  library(dplyr)
  library(purrr)
})

# Read the overall config file.
config <- yaml::read_yaml(here::here("config.yaml"))

scran_pars <- expand_grid(
  method = "scran",
  test.type = c("t", "wilcox", "binom"),
  pval.type = c("any", "all")
) %>%
  mutate(file_name = paste0(method, "_", test.type, "_", pval.type))

seurat_pars <- expand_grid(
  method = "seurat",
  test.use = c("wilcox", "bimod", "roc", "t", "negbinom", "poisson", "LR",
               "MAST", "DESeq2")
) %>%
  mutate(file_name = paste0(method, "_", test.use))

method_pars_data <- list(
  scran_pars,
  seurat_pars
)

method_pars <- flatten(map(method_pars_data, ~ transpose(.x, .names = .x$file_name)))
method_ids <- names(method_pars)

method_paths <- paste0(here::here(config$method_pars_folder, method_ids), ".json")
names(method_paths) <- method_ids

# Write all the parameters of each method to file.
# The checking of previous parameters is needed so that we don't constantly
# make the files newer, breaking Snakemake.
# FIXME: How can we clean up out of date parameter files?
for (id in method_ids) {
  new <- method_pars[[id]]
  path <- method_paths[[id]]

  if (file.exists(path)) {
    old <- jsonlite::fromJSON(path)
    if (!identical(old, new)) {
      # The file already exists, but we changed it.
      write(jsonlite::toJSON(new, null = "null"), path)
    }
  } else {
    # This file doesn't already exist.
    write(jsonlite::toJSON(new, null = "null"), path)
  }
}
